// @ts-nocheck
import { Component } from '@angular/core';
import {Override, TaskModel, DependencyType, TimeUnit } from '@bryntum/gantt';

@Component({
    selector      : 'app-root',
    templateUrl   : './app.component.html',
    styleUrls     : ['./app.component.scss']
//    encapsulation : ViewEncapsulation.None
})

export class AppComponent {
    ngAfterViewInit() {
        // EDGE-399
        Override.apply(class {
            // static target = {
            //     class: TaskModel
            // }
            static get target() {
                return {class:TaskModel}
            }
            // Our overriden implementation here...
            *calculateEarlyEndDateConstraintIntervals() {
                // const superProto = this.constructor.prototype
                // const intervals = yield* superProto.calculateEarlyEndDateConstraintIntervals.call(this);
                const intervals = yield* this._overridden.calculateEarlyEndDateConstraintIntervals.call(this);
                const project = this.getProject();
                const dependencyConstraintIntervalClass = project.dependencyConstraintIntervalClass;
                for (const dependency of (yield this.$.incomingDeps)) {
                    // ignore missing from events and inactive dependencies
                    if (!(yield* this.shouldPredecessorAffectScheduling(dependency)))
                        continue;
                    const predecessor = yield dependency.$.fromEvent;
                    let predecessorDate;
                    switch (yield dependency.$.type) {
                        case DependencyType.EndToEnd:
                            predecessorDate = yield predecessor.$.earlyEndDateRaw;
                            break;
                        case DependencyType.StartToEnd:
                            predecessorDate = yield predecessor.$.earlyStartDateRaw;
                            break;
                    }
                    if (predecessorDate) {
                        const lag = yield dependency.$.lag;
                        const lagUnit = yield dependency.$.lagUnit;
                        const calendar = yield dependency.$.calendar;
                        // this "subscribes" on the calendar's `version` field (which is incremented
                        // every time when the intervals of the calendar changes)
                        yield calendar.$.version;
                        let endDate = calendar.calculateEndDate(
                            predecessorDate,
                            yield* project.$convertDuration(lag, lagUnit, TimeUnit.Millisecond)
                        );
                        let endDateNonWorkingTime = calendar.skipNonWorkingTime(
                            endDate,
                            lag >= 0);
                            console.log("endDate endDateNonWorkingTime lag");
                        console.log(endDate, endDateNonWorkingTime, lag);
                        const interval = dependencyConstraintIntervalClass.new({
                            owner: dependency,
                            //startDate: calendar.calculateEndDate(predecessorDate, yield* project.$convertDuration(lag, lagUnit, TimeUnit.Millisecond)),
                            startDate   : endDateNonWorkingTime,
                            endDate: null
                        });
                        intervals.unshift(interval);
                    }
                }
                return intervals;
            }
        })
    }
}
